<?php

/**
 * @license LGPLv3, http://opensource.org/licenses/LGPL-3.0
 * @copyright Aimeos (aimeos.org), 2015
 * @package Client
 * @subpackage Html
 */


/**
 * Default implementation of the account creation e-mails
 *
 * @package Client
 * @subpackage Html
 */
class Client_Html_Email_Account_Default
	extends Client_Html_Common_Client_Factory_Abstract
	implements Client_Html_Common_Client_Factory_Interface
{
	/** client/html/email/account/default/subparts
	 * List of HTML sub-clients rendered within the account creation e-mail
	 *
	 * The output of the frontend is composed of the code generated by the HTML
	 * clients. Each HTML client can consist of serveral (or none) sub-clients
	 * that are responsible for rendering certain sub-parts of the output. The
	 * sub-clients can contain HTML clients themselves and therefore a
	 * hierarchical tree of HTML clients is composed. Each HTML client creates
	 * the output that is placed inside the container of its parent.
	 *
	 * At first, always the HTML code generated by the parent is printed, then
	 * the HTML code of its sub-clients. The order of the HTML sub-clients
	 * determines the order of the output of these sub-clients inside the parent
	 * container. If the configured list of clients is
	 *
	 *  array( "subclient1", "subclient2" )
	 *
	 * you can easily change the order of the output by reordering the subparts:
	 *
	 *  client/html/<clients>/subparts = array( "subclient1", "subclient2" )
	 *
	 * You can also remove one or more parts if they shouldn't be rendered:
	 *
	 *  client/html/<clients>/subparts = array( "subclient1" )
	 *
	 * As the clients only generates structural HTML, the layout defined via CSS
	 * should support adding, removing or reordering content by a fluid like
	 * design.
	 *
	 * @param array List of sub-client names
	 * @since 2015.09
	 * @category Developer
	 */
	private $_subPartPath = 'client/html/email/account/default/subparts';

	/** client/html/email/account/text/name
	 * Name of the text part used by the account creation e-mail client implementation
	 *
	 * Use "Myname" if your class is named "Client_Html_Email_Account_Text_Myname".
	 * The name is case-sensitive and you should avoid camel case names like "MyName".
	 *
	 * @param string Last part of the client class name
	 * @since 2015.09
	 * @category Developer
	 */

	/** client/html/email/account/html/name
	 * Name of the html part used by the account creation e-mail client implementation
	 *
	 * Use "Myname" if your class is named "Client_Html_Email_Account_Html_Myname".
	 * The name is case-sensitive and you should avoid camel case names like "MyName".
	 *
	 * @param string Last part of the client class name
	 * @since 2015.09
	 * @category Developer
	 */
	private $_subPartNames = array( 'text', 'html' );


	/**
	 * Returns the HTML code for insertion into the body.
	 *
	 * @param string $uid Unique identifier for the output if the content is placed more than once on the same page
	 * @param array &$tags Result array for the list of tags that are associated to the output
	 * @param string|null &$expire Result variable for the expiration date of the output (null for no expiry)
	 * @return string HTML code
	 */
	public function getBody( $uid = '', array &$tags = array(), &$expire = null )
	{
		$view = $this->_setViewParams( $this->getView(), $tags, $expire );

		$content = '';
		foreach( $this->_getSubClients() as $subclient ) {
			$content .= $subclient->setView( $view )->getBody( $uid, $tags, $expire );
		}
		$view->accountBody = $content;

		/** client/html/email/account/default/template-body
		 * Relative path to the HTML body template of the account creation e-mail client.
		 *
		 * The template file contains the HTML code and processing instructions
		 * to generate the result shown in the body of the frontend. The
		 * configuration string is the path to the template file relative
		 * to the layouts directory (usually in client/html/layouts).
		 *
		 * You can overwrite the template file configuration in extensions and
		 * provide alternative templates. These alternative templates should be
		 * named like the default one but with the string "default" replaced by
		 * an unique name. You may use the name of your project for this. If
		 * you've implemented an alternative client class as well, "default"
		 * should be replaced by the name of the new class.
		 *
		 * The product notification e-mail HTML client allows to use a different template for
		 * each account status value. You can create a template for each account
		 * status and store it in the "email/account/<status number>/" directory
		 * below the "layouts" directory (usually in client/html/layouts). If no
		 * specific layout template is found, the common template in the
		 * "email/account/" directory is used.
		 *
		 * @param string Relative path to the template creating code for the HTML page body
		 * @since 2015.09
		 * @category Developer
		 * @see client/html/email/account/default/template-header
		 */
		$tplconf = 'client/html/email/account/default/template-body';

		return $view->render( $this->_getTemplate( $tplconf, 'email/account/body-default.html' ) );
	}


	/**
	 * Returns the HTML string for insertion into the header.
	 *
	 * @param string $uid Unique identifier for the output if the content is placed more than once on the same page
	 * @param array &$tags Result array for the list of tags that are associated to the output
	 * @param string|null &$expire Result variable for the expiration date of the output (null for no expiry)
	 * @return string|null String including HTML tags for the header on error
	 */
	public function getHeader( $uid = '', array &$tags = array(), &$expire = null )
	{
		$view = $this->_setViewParams( $this->getView(), $tags, $expire );

		$content = '';
		foreach( $this->_getSubClients() as $subclient ) {
			$content .= $subclient->setView( $view )->getHeader( $uid, $tags, $expire );
		}
		$view->accountHeader = $content;


		$addr = $view->extAddressItem;

		$msg = $view->mail();
		$msg->addHeader( 'X-MailGenerator', 'Aimeos' );
		$msg->addTo( $addr->getEMail(), $addr->getFirstName() . ' ' . $addr->getLastName() );


		/** client/html/email/from-name
		 * @see client/html/email/account/from-email
		 */
		$fromName = $view->config( 'client/html/email/from-name' );

		/** client/html/email/account/from-name
		 * Name used when sending account creation e-mails
		 *
		 * The name of the person or e-mail account that is used for sending all
		 * shop related account e-mails to customers. This configuration option
		 * overwrite the name set in "client/html/email/from-name".
		 *
		 * @param string Name shown in the e-mail
		 * @since 2015.09
		 * @category User
		 * @see client/html/email/from-name
		 * @see client/html/email/from-email
		 * @see client/html/email/reply-email
		 * @see client/html/email/bcc-email
		 */
		$fromNameAccount = $view->config( 'client/html/email/account/from-name', $fromName );

		/** client/html/email/from-email
		 * @see client/html/email/account/from-email
		 */
		$fromEmail = $view->config( 'client/html/email/from-email' );

		/** client/html/email/account/from-email
		 * E-Mail address used when sending account creation e-mails
		 *
		 * The e-mail address of the person or account that is used for sending
		 * all shop related product notification e-mails to customers. This configuration option
		 * overwrites the e-mail address set via "client/html/email/from-email".
		 *
		 * @param string E-mail address
		 * @since 2015.09
		 * @category User
		 * @see client/html/email/account/from-name
		 * @see client/html/email/from-email
		 * @see client/html/email/reply-email
		 * @see client/html/email/bcc-email
		 */
		if( ( $fromEmailAccount = $view->config( 'client/html/email/account/from-email', $fromEmail ) ) != null ) {
			$msg->addFrom( $fromEmailAccount, $fromNameAccount );
		}


		/** client/html/email/reply-name
		 * @see client/html/email/account/reply-email
		 */
		$replyName = $view->config( 'client/html/email/reply-name', $fromName );

		/** client/html/email/account/reply-name
		 * Recipient name displayed when the customer replies to account creation e-mails
		 *
		 * The name of the person or e-mail account the customer should
		 * reply to in case of account related questions or problems. This
		 * configuration option overwrites the name set via
		 * "client/html/email/reply-name".
		 *
		 * @param string Name shown in the e-mail
		 * @since 2015.09
		 * @category User
		 * @see client/html/email/account/reply-email
		 * @see client/html/email/reply-name
		 * @see client/html/email/reply-email
		 * @see client/html/email/from-email
		 * @see client/html/email/bcc-email
		 */
		$replyNameAccount = $view->config( 'client/html/email/account/reply-name', $replyName );

		/** client/html/email/reply-email
		 * @see client/html/email/account/reply-email
		 */
		$replyEmail = $view->config( 'client/html/email/reply-email', $fromEmail );

		/** client/html/email/account/reply-email
		 * E-Mail address used by the customer when replying to account creation e-mails
		 *
		 * The e-mail address of the person or e-mail account the customer
		 * should reply to in case of account related questions or problems.
		 * This configuration option overwrites the e-mail address set via
		 * "client/html/email/reply-email".
		 *
		 * @param string E-mail address
		 * @since 2015.09
		 * @category User
		 * @see client/html/email/account/reply-name
		 * @see client/html/email/reply-email
		 * @see client/html/email/from-email
		 * @see client/html/email/bcc-email
		 */
		if( ( $replyEmailAccount = $view->config( 'client/html/email/account/reply-email', $replyEmail ) ) != null ) {
			$msg->addReplyTo( $replyEmailAccount, $replyNameAccount );
		}


		/** client/html/email/bcc-email
		 * @see client/html/email/account/bcc-email
		 */
		$bccEmail = $view->config( 'client/html/email/bcc-email' );

		/** client/html/email/account/bcc-email
		 * E-Mail address all account creation e-mails should be also sent to
		 *
		 * Using this option you can send a copy of all account related e-mails
		 * to a second e-mail account. This can be handy for testing and checking
		 * the e-mails sent to customers.
		 *
		 * It also allows shop owners with a very small volume of orders to be
		 * notified about account changes. Be aware that this isn't useful if the
		 * order volumne is high or has peeks!
		 *
		 * This configuration option overwrites the e-mail address set via
		 * "client/html/email/bcc-email".
		 *
		 * @param string E-mail address
		 * @since 2015.09
		 * @category User
		 * @category Developer
		 * @see client/html/email/bcc-email
		 * @see client/html/email/reply-email
		 * @see client/html/email/from-email
		 */
		if( ( $bccEmailAccount = $view->config( 'client/html/email/account/bcc-email', $bccEmail ) ) != null ) {
			$msg->addBcc( $bccEmailAccount );
		}


		/** client/html/email/account/default/template-header
		 * Relative path to the HTML header template of the account creation e-mail client.
		 *
		 * The template file contains the HTML code and processing instructions
		 * to generate the HTML code that is inserted into the HTML page header
		 * of the rendered page in the frontend. The configuration string is the
		 * path to the template file relative to the layouts directory (usually
		 * in client/html/layouts).
		 *
		 * You can overwrite the template file configuration in extensions and
		 * provide alternative templates. These alternative templates should be
		 * named like the default one but with the string "default" replaced by
		 * an unique name. You may use the name of your project for this. If
		 * you've implemented an alternative client class as well, "default"
		 * should be replaced by the name of the new class.
		 *
		 * The product notification e-mail HTML client allows to use a different template for
		 * each account status value. You can create a template for each account
		 * status and store it in the "email/account/<status number>/" directory
		 * below the "layouts" directory (usually in client/html/layouts). If no
		 * specific layout template is found, the common template in the
		 * "email/account/" directory is used.
		 *
		 * @param string Relative path to the template creating code for the HTML page head
		 * @since 2015.09
		 * @category Developer
		 * @see client/html/email/account/default/template-body
		 */
		$tplconf = 'client/html/email/account/default/template-header';

		return $view->render( $this->_getTemplate( $tplconf, 'email/account/header-default.html' ) ); ;
	}


	/**
	 * Returns the sub-client given by its name.
	 *
	 * @param string $type Name of the client type
	 * @param string|null $name Name of the sub-client (Default if null)
	 * @return Client_Html_Interface Sub-client object
	 */
	public function getSubClient( $type, $name = null )
	{
		/** client/html/email/account/decorators/excludes
		 * Excludes decorators added by the "common" option from the email account html client
		 *
		 * Decorators extend the functionality of a class by adding new aspects
		 * (e.g. log what is currently done), executing the methods of the underlying
		 * class only in certain conditions (e.g. only for logged in users) or
		 * modify what is returned to the caller.
		 *
		 * This option allows you to remove a decorator added via
		 * "client/html/common/decorators/default" before they are wrapped
		 * around the html client.
		 *
		 *  client/html/email/account/decorators/excludes = array( 'decorator1' )
		 *
		 * This would remove the decorator named "decorator1" from the list of
		 * common decorators ("Client_Html_Common_Decorator_*") added via
		 * "client/html/common/decorators/default" to the html client.
		 *
		 * @param array List of decorator names
		 * @since 2015.09
		 * @category Developer
		 * @see client/html/common/decorators/default
		 * @see client/html/email/account/decorators/global
		 * @see client/html/email/account/decorators/local
		 */

		/** client/html/email/account/decorators/global
		 * Adds a list of globally available decorators only to the email account html client
		 *
		 * Decorators extend the functionality of a class by adding new aspects
		 * (e.g. log what is currently done), executing the methods of the underlying
		 * class only in certain conditions (e.g. only for logged in users) or
		 * modify what is returned to the caller.
		 *
		 * This option allows you to wrap global decorators
		 * ("Client_Html_Common_Decorator_*") around the html client.
		 *
		 *  client/html/email/account/decorators/global = array( 'decorator1' )
		 *
		 * This would add the decorator named "decorator1" defined by
		 * "Client_Html_Common_Decorator_Decorator1" only to the html client.
		 *
		 * @param array List of decorator names
		 * @since 2015.09
		 * @category Developer
		 * @see client/html/common/decorators/default
		 * @see client/html/email/account/decorators/excludes
		 * @see client/html/email/account/decorators/local
		 */

		/** client/html/email/account/decorators/local
		 * Adds a list of local decorators only to the email account html client
		 *
		 * Decorators extend the functionality of a class by adding new aspects
		 * (e.g. log what is currently done), executing the methods of the underlying
		 * class only in certain conditions (e.g. only for logged in users) or
		 * modify what is returned to the caller.
		 *
		 * This option allows you to wrap local decorators
		 * ("Client_Html_Email_Decorator_*") around the html client.
		 *
		 *  client/html/email/account/decorators/local = array( 'decorator2' )
		 *
		 * This would add the decorator named "decorator2" defined by
		 * "Client_Html_Email_Decorator_Decorator2" only to the html client.
		 *
		 * @param array List of decorator names
		 * @since 2015.09
		 * @category Developer
		 * @see client/html/common/decorators/default
		 * @see client/html/email/account/decorators/excludes
		 * @see client/html/email/account/decorators/global
		 */

		return $this->_createSubClient( 'email/account/' . $type, $name );
	}


	/**
	 * Returns the list of sub-client names configured for the client.
	 *
	 * @return array List of HTML client names
	 */
	protected function _getSubClientNames()
	{
		return $this->_getContext()->getConfig()->get( $this->_subPartPath, $this->_subPartNames );
	}
}